package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.CentralizedRequestDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.CentralizedRequestMapper;
import digital.laboratory.platform.reagent.mapper.PurchasingPlanMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import feign.Response;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * (集中采购申请)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (集中采购申请) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class CentralizedRequestServiceImpl extends ServiceImpl<CentralizedRequestMapper, CentralizedRequest> implements CentralizedRequestService {
    @Autowired
    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Autowired
    private ProcurementContentService procurementContentService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired

    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;

    @Autowired
    private OssFile ossFile;
    @Autowired

    private RemoteWord2PDFService remoteWord2PDFService;


    @Transactional
    @Override//增加集中采购申请
    public CentralizedRequest addRequest(List<CentralizedRequestDTO> centralizedRequestDTOList, DLPUser dlpUser) {

        String centralizedRequestId = centralizedRequestDTOList.get(0).getCentralizedRequestId();

        List<DetailsOfCentralized> detailsOfCentralizedList = new ArrayList<>();

        if (centralizedRequestId == null) {

            CentralizedRequest centralizedRequest = new CentralizedRequest();

            centralizedRequest.setApplicantId(dlpUser.getId());

            centralizedRequest.setId(IdWorker.get32UUID().toUpperCase());
            //获取申请时间
            //未提交
            centralizedRequest.setStatus(0);

            centralizedRequest.setApplicantName(dlpUser.getName());

            for (CentralizedRequestDTO centralizedRequestDTO : centralizedRequestDTOList) {

                DetailsOfCentralized detailsOfCentralized = new DetailsOfCentralized();

                BeanUtils.copyProperties(centralizedRequestDTO, detailsOfCentralized);

                detailsOfCentralized.setId(IdWorker.get32UUID().toUpperCase());

                detailsOfCentralized.setCentralizedRequestId(centralizedRequest.getId());

                detailsOfCentralizedList.add(detailsOfCentralized);


            }

            if (this.save(centralizedRequest) && detailsOfCentralizedService.saveBatch(detailsOfCentralizedList)) {
                return centralizedRequest;
            } else return null;


            //将DTO赋值给明细表


        } else {

            CentralizedRequest centralizedRequest = this.getById(centralizedRequestId);


            for (CentralizedRequestDTO centralizedRequestDTO : centralizedRequestDTOList) {

                DetailsOfCentralized detailsOfCentralized = new DetailsOfCentralized();

                BeanUtils.copyProperties(centralizedRequestDTO, detailsOfCentralized);

                detailsOfCentralized.setId(IdWorker.get32UUID().toUpperCase());

                detailsOfCentralized.setCentralizedRequestId(centralizedRequest.getId());

                detailsOfCentralizedList.add(detailsOfCentralized);

            }

            if (detailsOfCentralizedService.saveBatch(detailsOfCentralizedList)) {

                return centralizedRequest;

            } else return null;
        }
    }


    @Override//提交申请
    public CentralizedRequest commitRequest(String id, DLPUser dlpUser) {
        CentralizedRequest byId = this.getById(id);
        byId.setStatus(1);
        byId.setDateOfApplication(LocalDateTime.now());
        this.updateById(byId);
        return byId;
    }


    @Override//修改申请
    public DetailsOfCentralized editDetailsById(CentralizedRequestDTO centralizedRequestDto) {

        DetailsOfCentralized detailsOfCentralized = detailsOfCentralizedService.getById(centralizedRequestDto.getDetailsOfCentralizedId());

        BeanUtils.copyProperties(centralizedRequestDto, detailsOfCentralized);

        if (detailsOfCentralizedService.updateById(detailsOfCentralized)) {
            return detailsOfCentralized;
        } else return null;

    }

    @Override//增加明细
    public DetailsOfCentralized addDetailsById(CentralizedRequestDTO centralizedRequestDto) {

        DetailsOfCentralized detailsOfCentralized = new DetailsOfCentralized();

        detailsOfCentralized.setId(IdWorker.get32UUID().toUpperCase());

        BeanUtils.copyProperties(centralizedRequestDto, detailsOfCentralized);

        if (detailsOfCentralizedService.save(detailsOfCentralized)) {
            return detailsOfCentralized;
        } else return null;
    }

    @Override//删除申请
    @Transactional
    public Boolean delRequestById(String centralizedRequestId) {

        LambdaQueryWrapper<DetailsOfCentralized> detailsOfCentralizedLambdaQueryWrapper = new LambdaQueryWrapper<>();

        detailsOfCentralizedLambdaQueryWrapper.eq(DetailsOfCentralized::getCentralizedRequestId, centralizedRequestId);

        List<DetailsOfCentralized> list = detailsOfCentralizedService.list(detailsOfCentralizedLambdaQueryWrapper);

        if (list.size() != 0) {

            detailsOfCentralizedService.removeBatchByIds(list);

        }
        return this.removeById(centralizedRequestId);

    }


    @Override
    public IPage<CentralizedRequestVO> getCentralizedRequestVOPage(IPage<CentralizedRequest> page, QueryWrapper<CentralizedRequest> qw) {
        IPage<CentralizedRequestVO> r = baseMapper.getCentralizedRequestVOPage(page, qw);
        return r;
    }

    @Override
    public List<CentralizedRequestVO> getCentralizedRequestVOList(QueryWrapper<CentralizedRequest> qw) {

        LambdaQueryWrapper<CentralizedRequest> centralizedRequestLambdaQueryWrapper = new LambdaQueryWrapper<>();
        //查询状态为1的采购申请：已提交
        centralizedRequestLambdaQueryWrapper.eq(CentralizedRequest::getStatus, 1);

        List<CentralizedRequest> list = this.list(centralizedRequestLambdaQueryWrapper);

        ArrayList<CentralizedRequestVO> centralizedRequestVOArrayList = new ArrayList<>();

        for (CentralizedRequest centralizedRequest : list) {
            CentralizedRequestVO centralizedRequestVO = new CentralizedRequestVO();
            BeanUtils.copyProperties(centralizedRequest, centralizedRequestVO);
            centralizedRequestVO.setDetailsOfCentralizedVOList(detailsOfCentralizedService.getDetailsOfCentralizedVOList(centralizedRequestVO.getId()));
            centralizedRequestVOArrayList.add(centralizedRequestVO);
        }

        return centralizedRequestVOArrayList;
    }

    @Override//
    public CentralizedRequestVO getCentralizedRequestVO(String centralizedRequestId) {

        CentralizedRequestVO centralizedRequestVO = baseMapper.getCentralizedRequestVO(centralizedRequestId);

        List<DetailsOfCentralizedVO> detailsOfCentralizedVOList = detailsOfCentralizedService.getDetailsOfCentralizedVOList(centralizedRequestId);

        centralizedRequestVO.setDetailsOfCentralizedVOList(detailsOfCentralizedVOList);

        return centralizedRequestVO;
    }

    @Override
    public List<CentralizedRequestVO> getVOList(QueryWrapper<CentralizedRequest> qw) {

        List<CentralizedRequestVO> centralizedRequestVOArrayList = baseMapper.getVOList(qw);

//        for (CentralizedRequestVO centralizedRequestVO : centralizedRequestVOArrayList) {
//
//            List<DetailsOfCentralizedVO> detailsOfCentralizedVOList = detailsOfCentralizedService.getDetailsOfCentralizedVOList(centralizedRequestVO.getId());
//
//            centralizedRequestVO.setDetailsOfCentralizedVOList(detailsOfCentralizedVOList);
//        }

        return centralizedRequestVOArrayList;
    }

    @Override
    public CentralizedRequest auditById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        CentralizedRequest byId = this.getById(auditAndApproveDTO.getUuId());

        byId.setAuditId(dlpUser.getId());
        byId.setAuditTime(LocalDateTime.now());
        byId.setAuditOpinion(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResult(auditAndApproveDTO.getAuditResult());

        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setStatus(6);
        } else {
            byId.setStatus(-2);
        }
        if (this.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override
    public List<PurchaseRequestPrintVO> getPurchaseRequestPrintList(String id, String type) {
        ArrayList<PurchaseRequestPrintVO> purchaseRequestPrintVOList = new ArrayList<>();

        if (type.equals("集中采购申请")) {

            CentralizedRequestVO centralizedRequest = this.getCentralizedRequestVO(id);

            if (centralizedRequest.getStatus() != 4) {

                throw new RuntimeException(String.format("当前采购还未被制定为清单，无法打印"));
            }

            List<DetailsOfCentralizedVO> detailsOfCentralizedVOList = centralizedRequest.getDetailsOfCentralizedVOList();


            for (DetailsOfCentralizedVO detailsOfCentralized : detailsOfCentralizedVOList) {

                ReagentConsumables reagentConsumables = reagentConsumablesService.getById(detailsOfCentralized.getReagentConsumableId());

                if (reagentConsumables.getCategory().equals("标准物质")) {

                    PurchaseRequestPrintVO purchaseRequestPrintVO = new PurchaseRequestPrintVO();
                    purchaseRequestPrintVO.setApplicantName(centralizedRequest.getApplicantName());
                    purchaseRequestPrintVO.setPurpose(detailsOfCentralized.getPurpose());
                    purchaseRequestPrintVO.setQuantity(detailsOfCentralized.getQuantityPurchased());
                    purchaseRequestPrintVO.setRemarks(detailsOfCentralized.getRemarks());
                    purchaseRequestPrintVO.setReagentConsumableName(reagentConsumables.getReagentConsumableName());
                    purchaseRequestPrintVO.setSpecificationAndModel(reagentConsumables.getSpecificationAndModel());
                    purchaseRequestPrintVO.setUnitPrice(procurementContentService.getById(detailsOfCentralized.getProcurementContentId()).getUnitPrice());
                    purchaseRequestPrintVO.setSubtotal(procurementContentService.getById(detailsOfCentralized.getProcurementContentId()).getSubtotal());
                    purchaseRequestPrintVO.setSupplierName(supplierInformationService.getById(purchaseListDetailsService.getOne(Wrappers.<PurchaseListDetails>query().eq("procurement_content_id", detailsOfCentralized.getProcurementContentId())).getSupplierId()).getSupplierName());
                    purchaseRequestPrintVOList.add(purchaseRequestPrintVO);
                }

            }
        } else {

            DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(id);

            if (requestById.getStatus() != 6) {
                throw new RuntimeException(String.format("当前采购还未被制定为清单，无法打印"));
            }

            for (DecentralizeDetails decentralizeDetails : requestById.getDecentralizeDetailsVOList()) {

                if (decentralizeDetails.getCategory().equals("标准物质")) {

                    PurchaseRequestPrintVO purchaseRequestPrintVO = new PurchaseRequestPrintVO();

                    BeanUtils.copyProperties(decentralizeDetails, purchaseRequestPrintVO);

                    purchaseRequestPrintVO.setPurpose(decentralizeDetails.getPurpose());
                    purchaseRequestPrintVO.setApplicantName(requestById.getApplicantName());
                    purchaseRequestPrintVO.setRemarks(decentralizeDetails.getRemarks());

                    purchaseRequestPrintVO.setSubtotal(purchaseRequestPrintVO.getUnitPrice() * purchaseRequestPrintVO.getQuantity());
                    //查找出供应商名称
                    purchaseRequestPrintVO.setSupplierName(supplierInformationService.getById(purchaseListDetailsService.getById(decentralizeDetails.getPurchaseListDetailsId()).getSupplierId()).getSupplierName());

                    purchaseRequestPrintVOList.add(purchaseRequestPrintVO);
                }
            }
        }
        return purchaseRequestPrintVOList;
    }

    @Override
    public void purchaseRequestTablePDF(List<PurchaseRequestPrintVO> voList, String id, String type, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("purchaseRequestTablePDF.................");

        int x = 1;

        double total = 0;

        for (PurchaseRequestPrintVO purchaseRequestPrintVO : voList) {
            purchaseRequestPrintVO.setI(x);
            total = total + purchaseRequestPrintVO.getSubtotal();
            x++;
        }

        String firstAuditName = null;
        String secondAuditName = null;
        String thirdAuditName = null;
        String fourthAuditName = null;
        String commitTime = null;

        //-----------------------------
        // 生成 word 版本的 采购目录

        if (type.equals("集中采购申请")) {

            CentralizedRequestVO centralizedRequestVO = this.getCentralizedRequestVO(id);
            if (centralizedRequestVO.getStatus() != 4) {
                throw new RuntimeException(String.format("还未被制定为采购清单，无法打印"));
            }
            PurchasingPlanVO byId = purchasingPlanService.getPurchasingPlanVO(centralizedRequestVO.getPurchasingPlanId());
            firstAuditName = centralizedRequestVO.getAuditorName();
            secondAuditName = byId.getCreateName();
            thirdAuditName = byId.getPrimaryAuditorName();
            fourthAuditName = byId.getApproverName();
            commitTime = LocalDateTimeUtil.format(centralizedRequestVO.getDateOfApplication(), "yyyy年MM月dd日");

        } else {
            DecentralizedRequestVO requestById = decentralizedRequestService.getRequestById(id);
            if (requestById.getStatus() != 6) {
                throw new RuntimeException(String.format("还未被制定为采购清单，无法打印"));
            }
            firstAuditName = requestById.getPrimaryAuditorName();
            secondAuditName = requestById.getSecondaryAuditorName();
            thirdAuditName = requestById.getThreeLevelAuditName();
            fourthAuditName = requestById.getApproverName();
            commitTime = LocalDateTimeUtil.format(requestById.getCommitTime(), "yyyy年MM月dd日");
        }
        String applyFileName = "标准物质采购申请表-" + id;

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ossFile.fileGet("template" + "/" + "标准物质采购申请表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);
        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);
        builder.bind("voList", policy).build();

        /**
         * 循环表格
         */
        double finalTotal = total;
        String finalFourthAuditName = fourthAuditName;
        String finalSecondAuditName = secondAuditName;
        String finalThirdAuditName = thirdAuditName;
        String finalFirstAuditName = firstAuditName;
        String finalCommitTime = commitTime;
        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);

        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("voList", voList);
                    put("year", year);
                    put("total", finalTotal);
                    put("finalFirstAuditName", finalFirstAuditName);
                    put("finalSecondAuditName", finalSecondAuditName);
                    put("finalThirdAuditName", finalThirdAuditName);
                    put("finalFourthAuditName", finalFourthAuditName);
                    put("commitTime", finalCommitTime);
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();

        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();

        ossFile.fileSave("document" + "/" + "purchaseRequest" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();

        System.out.println(String.format("转换为 PDF 结束"));
    }
}



