package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.ComplianceCheckDTO;
import digital.laboratory.platform.reagent.dto.OutgoingContentsDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.DeliveryRegistrationFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.DeliveryRegistrationFormVO;
import digital.laboratory.platform.reagent.vo.OutgoingContentsVO;
import digital.laboratory.platform.reagent.vo.ReagentConsumablesSetVO;
import digital.laboratory.platform.sys.entity.CellAndStoreSupplyData;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
import io.seata.spring.annotation.GlobalTransactional;
import io.swagger.models.auth.In;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * (试剂耗材出库登记表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材出库登记表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class DeliveryRegistrationFormServiceImpl extends ServiceImpl<DeliveryRegistrationFormMapper, DeliveryRegistrationForm> implements DeliveryRegistrationFormService {

    @Autowired
    private ApplicationForUseService applicationForUseService;
    @Autowired
    private ReagentConsumablesSetService reagentConsumablesSetService;
    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private OutgoingContentsService outgoingContentsService;
    @Autowired
    private ComplianceCheckService complianceCheckService;
    @Autowired
    private StandardMaterialApplicationService standardMaterialApplicationService;
    @Autowired
    private RequisitionRecordService requisitionRecordService;
    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;
    @Autowired
    private SolutionUseFormService solutionUseFormService;
    @Autowired
    private ReferenceMaterialService referenceMaterialService;
    @Autowired
    private BatchDetailsService batchDetailsService;
    @Autowired
    private StandardReserveSolutionService standardReserveSolutionService;
    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;

    @Autowired
    private ReagentConsumableStashService reagentConsumableStashService;


    @Override
    @Transactional//创建出库任务
    public DeliveryRegistrationForm addFrom(String applicationForUseId) {

        ApplicationForUse applicationForUse = applicationForUseService.getById(applicationForUseId);//领用申请表
        //创建出库登记表
        DeliveryRegistrationForm deliveryRegistrationForm = new DeliveryRegistrationForm();

        deliveryRegistrationForm.setId(IdWorker.get32UUID().toUpperCase());
        deliveryRegistrationForm.setOutgoingApplicantId(applicationForUse.getRecipientId());//领用人~出库申请人
        deliveryRegistrationForm.setStatus(0);

        applicationForUse.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());//关联出库登记表
        //得到领用集合
        LambdaQueryWrapper<ReagentConsumablesSet> reagentConsumablesSetLambdaQueryWrapper = new LambdaQueryWrapper<>();
        reagentConsumablesSetLambdaQueryWrapper.eq(ReagentConsumablesSet::getApplicationForUseId, applicationForUse.getId());
        List<ReagentConsumablesSet> list = reagentConsumablesSetService.list(reagentConsumablesSetLambdaQueryWrapper);//领用申请明细

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();//出库明细集合
        //循环领用集合，创建出库集合
        for (ReagentConsumablesSet reagentConsumablesSet : list) {

            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(reagentConsumablesSet.getReagentConsumableId());//试剂耗材信息

            if (reagentConsumables.getCategory().equals("标准物质") | reagentConsumables.getCategory().equals("标准储备溶液")) {
                OutgoingContents outgoingContents = new OutgoingContents();//出库明细
                LambdaQueryWrapper<ReferenceMaterial> referenceMaterialLambdaQueryWrapper = new LambdaQueryWrapper<>();
                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(reagentConsumablesSet.getReferenceMaterialId());//标准物质
                outgoingContents.setReferenceMaterialId(referenceMaterial.getId());//标准物质ID
                outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
                outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());//关联出库登记表
                outgoingContents.setQuantity(reagentConsumablesSet.getQuantity());//领用申请数量
                outgoingContents.setRemarks(reagentConsumablesSet.getRemarks());//备注
                outgoingContents.setOutboundUse(reagentConsumablesSet.getPurpose());
                outgoingContents.setReagentConsumableId(reagentConsumablesSet.getReagentConsumableId());//试剂耗材ID
                outgoingContents.setReagentConsumableType(reagentConsumables.getCategory());//试剂耗材类别
                outgoingContents.setReferenceMaterialNumber(referenceMaterial.getNumber());//标准物质编号
                outgoingContents.setBatchDetailsId(reagentConsumablesSet.getBatchDetailsId());//批次ID
                outgoingContents.setLocation(referenceMaterial.getLocation());//位置信息
                outgoingContents.setLatticeId(referenceMaterial.getLatticeId());//格子ID
                outgoingContents.setBoxId(referenceMaterial.getBoxId());//柜子ID
                outgoingContents.setCode(referenceMaterial.getNumber());//扫码出库编号
                outgoingContents.setReagentConsumablesSetId(reagentConsumablesSet.getId());//关联领用申请明细

                outgoingContentsList.add(outgoingContents);//集合
            } else {
                String reagentConsumableId = reagentConsumablesSet.getReagentConsumableId();//试剂耗材ID
                //库存试剂耗材
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query().eq("reagent_consumable_id", reagentConsumableId));
                //所有批次集合
                List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_stash_id", reagentConsumablesSet.getReagentConsumableStashId())
                        .orderByAsc("create_time")
                        .ne("quantity", 0));

                //新建批次集合，用于存放需出库的物品批次
                List<BatchDetails> batchDetails = new ArrayList<>();
                //领用数量
                Integer quantity = reagentConsumablesSet.getQuantity();
                //数量，用于计算所需各个批次出库数量
                Integer batchQuantity = 0;

                for (int i = 0; i < batchDetailsList.size(); i++) {
                    //从最老的批次开始，如果这个批次的数量满足出库数量，则跳出循环
                    batchQuantity += batchDetailsList.get(i).getQuantity();
                    batchDetails.add(batchDetailsList.get(i));
                    if (batchQuantity >= quantity) {
                        break;
                    }
                }
                //循环应出库的对应所有批次
                for (int i = 0; i < batchDetails.size(); i++) {

                    OutgoingContents outgoingContents = new OutgoingContents();
                    outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
                    outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());//
                    outgoingContents.setQuantity(batchDetails.get(i).getQuantity());
                    outgoingContents.setRemarks(reagentConsumablesSet.getRemarks());
                    outgoingContents.setOutboundUse(reagentConsumablesSet.getPurpose());
                    outgoingContents.setReagentConsumableId(reagentConsumablesSet.getReagentConsumableId());
                    outgoingContents.setReagentConsumableType(reagentConsumables.getCategory());
                    outgoingContents.setBatchDetailsId(batchDetails.get(i).getBatchDetailsId());
                    outgoingContents.setLocation(batchDetails.get(i).getLocation());
                    outgoingContents.setLatticeId(batchDetails.get(i).getLatticeId());
                    outgoingContents.setBoxId(batchDetails.get(i).getBoxId());
                    outgoingContents.setCode(reagentConsumables.getCode());
                    outgoingContents.setReagentConsumablesSetId(reagentConsumablesSet.getId());
                    //遍历至最后一个批次时，要计算出这个批次该出库的数量
                    if (i == batchDetails.size() - 1) {
                        Integer x = quantity - (batchQuantity - batchDetailsList.get(i).getQuantity());
                        outgoingContents.setQuantity(x);
                    }
                    outgoingContentsList.add(outgoingContents);
                }
            }
        }
        if (this.save(deliveryRegistrationForm) && outgoingContentsService.saveBatch(outgoingContentsList) &
                applicationForUseService.updateById(applicationForUse)) {

            return deliveryRegistrationForm;
        } else throw new RuntimeException(String.format("创建出库任务失败"));
    }

    @Override//提交出库登记表
    @Transactional
    public DeliveryRegistrationForm commitForm(List<OutgoingContentsDTO> outgoingContentsDTOS, DLPUser dlpUser) {

        DeliveryRegistrationForm deliveryRegistrationForm = this.getById(outgoingContentsDTOS.get(0).getDeliveryRegistrationFormId());//得到出库登记表
        deliveryRegistrationForm.setStatus(1);//状态
        deliveryRegistrationForm.setDepositorId(dlpUser.getId());//出库人//

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, deliveryRegistrationForm.getId());
        //删除修改之前的内容(防止领用为X种，但实际出库只有n种，导致领用出库不一致)
        outgoingContentsService.removeBatchByIds(outgoingContentsService.list(outgoingContentsLambdaQueryWrapper));

        List<StandardMaterialApplication> standardMaterialApplications = new ArrayList<>();//标准物质领用归还登记表集合//

        List<OutgoingContents> outgoingContentsList = new ArrayList<>();//出库明细集合

        LambdaQueryWrapper<ApplicationForUse> applicationForUseLambdaQueryWrapper = new LambdaQueryWrapper<>();
        applicationForUseLambdaQueryWrapper.eq(ApplicationForUse::getDeliveryRegistrationFormId, deliveryRegistrationForm.getId());
        ApplicationForUse applicationForUse = applicationForUseService.getOne(applicationForUseLambdaQueryWrapper);//查询领用申请表

        applicationForUse.setClaimCode("");//要将领取码置空
        applicationForUse.setStatus(2);//状态：已出库

        applicationForUseService.updateById(applicationForUse);

        for (OutgoingContentsDTO outgoingContentsDTO : outgoingContentsDTOS) {
            //传入出库数量，区分出库数量与领用数量
            ReagentConsumablesSet reagentConsumablesSet = reagentConsumablesSetService.getById(outgoingContentsDTO.getReagentConsumablesSetId());
            //因为这里领取的一种物品，可能会对应多个批次，多个数量，但是对于领用人来说，只关心出库总数量
            if (reagentConsumablesSet.getQuantityDelivered() == null) {
                reagentConsumablesSet.setQuantityDelivered(outgoingContentsDTO.getQuantity());//
            } else {
                //一个批次对应一个出库明细，出库多个批次的情况下，要将每个批次的数量相加
                reagentConsumablesSet.setQuantityDelivered(reagentConsumablesSet.getQuantityDelivered() + outgoingContentsDTO.getQuantity());
            }
            reagentConsumablesSetService.updateById(reagentConsumablesSet);

            OutgoingContents outgoingContents = new OutgoingContents();
            BeanUtils.copyProperties(outgoingContentsDTO, outgoingContents);
            outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
            ReagentConsumables reagentConsumables = reagentConsumablesService.getById(outgoingContents.getReagentConsumableId());
            outgoingContents.setReagentConsumableType(reagentConsumables.getCategory());
            outgoingContents.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());
            outgoingContents.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());

            if (reagentConsumables.getCategory().equals("标准物质")) {
                //生成标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();

                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());//用途
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());//领用日期
                standardMaterialApplication.setRecipientId(applicationForUse.getRecipientId());//领用人
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());//试剂耗材类ID
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());//规格型号
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());//领用数量
                standardMaterialApplication.setApplicationForUseId(applicationForUse.getId());//关联领用申请ID
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());//标准物质ID
                standardMaterialApplication.setAdministratorId(dlpUser.getId());//出库人ID
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());//标准物质编号
                standardMaterialApplication.setIsReturn(outgoingContentsDTO.getOutboundUse());//是否用完
                standardMaterialApplication.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());//关联出库申请ID
                standardMaterialApplication.setReferenceSubstanceName(reagentConsumables.getReagentConsumableName());//试剂耗材名称

                standardMaterialApplications.add(standardMaterialApplication);
                if (outgoingContentsDTO.getOutboundUse() == 2) {//已用完，无需归还
                    standardMaterialApplication.setStatus(-1);
                    standardMaterialApplication.setPurposeAndQuantity(outgoingContents.getQuantity().doubleValue());
                } else {
                    standardMaterialApplication.setStatus(0);
                }
                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(outgoingContentsDTO.getReferenceMaterialId());

                if (referenceMaterial.getStatus() == -3) {
                    throw new RuntimeException(String.format("该物品已被其他工作人员领用"));
                }

                referenceMaterial.setStatus(-3);//改变状态
                referenceMaterial.setServiceStatus(-3);
//                reagentConsumableInventoryService.updateCabinet(referenceMaterial.getLatticeId(),referenceMaterial.getId(),-1);//清除这个格子上的物品信息
                referenceMaterialService.updateById(referenceMaterial);

                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());//批次信息

                if (batchDetails != null) {
                    batchDetails.setQuantity(batchDetails.getQuantity() - outgoingContentsDTO.getQuantity());//减少这个批次的数量
                    if (batchDetails.getQuantity() == 0) {//这个批次已被领用完
                        batchDetails.setServiceStatus(-1);
                    }
                    batchDetailsService.updateById(batchDetails);
                }

                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());
                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }
            } else if (reagentConsumables.getCategory().equals("标准储备溶液")) {
                StandardReserveSolution reference_id = standardReserveSolutionService.getOne(Wrappers.<StandardReserveSolution>query().eq("reference_id", outgoingContentsDTO.getReferenceMaterialId()));
                //创建标准物质领用/归还登记表
                StandardMaterialApplication standardMaterialApplication = new StandardMaterialApplication();
                standardMaterialApplication.setReferenceMaterialNumber(outgoingContentsDTO.getNumber());
                standardMaterialApplication.setStandardMaterialApplicationId(IdWorker.get32UUID().toUpperCase());
                standardMaterialApplication.setPurpose(outgoingContents.getOutboundUse());
                standardMaterialApplication.setDateOfClaim(LocalDateTime.now());
                standardMaterialApplication.setRecipientId(applicationForUse.getRecipientId());
                standardMaterialApplication.setReferenceSubstanceId(reagentConsumables.getReagentConsumableId());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setRequisitionedQuantity(outgoingContents.getQuantity());
                standardMaterialApplication.setSpecification(reagentConsumables.getSpecificationAndModel());
                standardMaterialApplication.setApplicationForUseId(applicationForUse.getId());
                standardMaterialApplication.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());
                standardMaterialApplication.setReferenceSubstanceName(reference_id.getSolutionName());

                if (outgoingContentsDTO.getOutboundUse() == 2) {
                    standardMaterialApplication.setPurposeAndQuantity(outgoingContents.getQuantity().doubleValue());
                    standardMaterialApplication.setStatus(1);
                } else {
                    standardMaterialApplication.setStatus(0);
                }
                standardMaterialApplication.setAdministratorId(dlpUser.getId());
                standardMaterialApplication.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                standardMaterialApplications.add(standardMaterialApplication);
                //录入标准储备溶液使用记录
                SolutionUseForm solutionUseForm = solutionUseFormService.useById(outgoingContentsDTO.getReferenceMaterialId(), dlpUser);
                solutionUseForm.setStandardMaterialApplicationId(standardMaterialApplication.getStandardMaterialApplicationId());//关联标准物质领用归还登记表
                solutionUseFormService.updateById(solutionUseForm);
                //通过标准物质ID，查找出对应的仓库信息，将库存量减少
                String referenceMaterialId = outgoingContentsDTO.getReferenceMaterialId();
                ReferenceMaterial referenceMaterial = referenceMaterialService.getById(referenceMaterialId);
                if (referenceMaterial.getStatus() == -3) {
                    throw new RuntimeException(String.format("该物品已被其他工作人员领用"));
                }
                referenceMaterial.setStatus(-3);
                referenceMaterial.setServiceStatus(-3);
                referenceMaterialService.updateById(referenceMaterial);
//                reagentConsumableInventoryService.updateCabinet(referenceMaterial.getLatticeId(),referenceMaterial.getId(),-1);//清除这个格子上的物品信息
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getById(referenceMaterial.getReagentConsumableInventoryId());
                reagentConsumableInventory.setTotalQuantity(reagentConsumableInventory.getTotalQuantity() - outgoingContentsDTO.getQuantity());//修改库存数量
                reagentConsumableInventoryService.updateById(reagentConsumableInventory);

                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());
                if (batchDetails != null) {
                    batchDetails.setQuantity(batchDetails.getQuantity() - outgoingContentsDTO.getQuantity());
                    if (batchDetails.getQuantity() == 0) {//这个批次已经被领用完
                        batchDetails.setServiceStatus(-1);
                    }
                    batchDetailsService.updateById(batchDetails);
                }
                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }
            } else {
                //如果不是标准物质
                RequisitionRecord requisitionRecord = requisitionRecordService.addRecord(outgoingContents);//试剂耗材领用申请表
                //查找对应的批次ID，将批次数量减少
                BatchDetails batchDetails = batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId());
                /*
                 * 若某个批次的物品已被领用完毕，则改变该批次的状态即可，就不会显示在领用页面*/
                if (batchDetails != null) {
                    //减少对应批次数量
                    batchDetails.setQuantity(batchDetails.getQuantity() - outgoingContentsDTO.getQuantity());
                    if (batchDetails.getQuantity() == 0) {
                        batchDetails.setServiceStatus(-1);
//                        reagentConsumableInventoryService.updateCabinet(batchDetails.getLatticeId(), reagentConsumables.getReagentConsumableId(),-1);
                    }
                    batchDetailsService.updateById(batchDetails);
                }
                ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getById(batchDetails.getReagentConsumableStashId());
                reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() - outgoingContentsDTO.getQuantity());
                reagentConsumableStashService.updateById(reagentConsumableStash);

                requisitionRecord.setDateOfClaim(LocalDateTime.now());//领用日期
                requisitionRecord.setRecipientId(applicationForUse.getRecipientId());//领用人ID
                requisitionRecord.setDeliveryRegistrationFormId(deliveryRegistrationForm.getId());//出库登记表ID
                requisitionRecordService.updateById(requisitionRecord);
                //查找出对应的仓库信息，将库存量减少
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.reduceById(outgoingContents.getReagentConsumableId(), outgoingContents.getQuantity());
                if (reagentConsumableInventory.getTotalQuantity() < 0) {
                    throw new RuntimeException(String.format("出库数量不能超过库存量"));
                }
            }
            //如果用途是用作案件，则需创建符合性检查记录表
            if (outgoingContents.getOutboundUse() == 0) {
                ComplianceCheckDTO complianceCheckDTO = new ComplianceCheckDTO();
                complianceCheckDTO.setBrand(reagentConsumables.getBrand());
                complianceCheckDTO.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                complianceCheckDTO.setSpecificationAndModel(reagentConsumables.getSpecificationAndModel());
                complianceCheckDTO.setComplianceCheckId(IdWorker.get32UUID().toUpperCase());
                complianceCheckDTO.setReagentConsumableNumber(outgoingContentsDTO.getNumber());
                complianceCheckDTO.setBatchDetailsId(batchDetailsService.getById(outgoingContentsDTO.getBatchDetailsId()).getBatchDetailsId());
                complianceCheckDTO.setReferenceMaterialId(outgoingContentsDTO.getReferenceMaterialId());
                complianceCheckService.addCheck(complianceCheckDTO, applicationForUse.getRecipientId());//新增符合性检查
            }
            outgoingContentsList.add(outgoingContents);
        }
        if (standardMaterialApplications.size() != 0) {
            standardMaterialApplicationService.saveBatch(standardMaterialApplications);
        }
        if (outgoingContentsService.saveBatch(outgoingContentsList) && this.updateById(deliveryRegistrationForm)) {
            return deliveryRegistrationForm;
        } else throw new RuntimeException(String.format("提交失败"));
    }

    @Override//分页查询
    public Page<DeliveryRegistrationFormVO> getDeliveryRegistrationFormVOPage(Page page, QueryWrapper<DeliveryRegistrationForm> qw) {
        Page deliveryRegistrationFormVOPage = baseMapper.getDeliveryRegistrationFormVOPage(page, qw);
        return deliveryRegistrationFormVOPage;
    }

    @Override//通过ID查询出库内容
    public DeliveryRegistrationFormVO getDeliveryRegistrationFormVOById(String deliveryRegistrationFormId) {
        //得到VO
        DeliveryRegistrationFormVO deliveryRegistrationFormVOById = baseMapper.getDeliveryRegistrationFormVOById(deliveryRegistrationFormId);

        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, deliveryRegistrationFormVOById.getId());
        //出库明细
        List<OutgoingContents> list = outgoingContentsService.list(outgoingContentsLambdaQueryWrapper);

        List<OutgoingContentsVO> outgoingContentsVOS = new ArrayList<>();
        if (list.size() != 0) {
            //为出库明细关联试剂耗材信息与供应商信息
            for (OutgoingContents outgoingContents : list) {

                OutgoingContentsVO outgoingContentsVO = new OutgoingContentsVO();
                BeanUtils.copyProperties(outgoingContents, outgoingContentsVO);
                LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();
                reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, outgoingContents.getReagentConsumableId());
                ReagentConsumableInventory reagentConsumableInventory = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);
                outgoingContentsVO.setReagentConsumableInventory(reagentConsumableInventory);//试剂耗材信息
                BatchDetails batchDetails = batchDetailsService.getById(outgoingContents.getBatchDetailsId());

                if (batchDetails != null) {
                    outgoingContentsVO.setBatch(batchDetails.getBatch());//批次
                    outgoingContentsVO.setSupplierName(supplierInformationService.getById(batchDetails.getSupplierId()).getSupplierName());//供应商名称
                }

                outgoingContentsVOS.add(outgoingContentsVO);
            }
            deliveryRegistrationFormVOById.setOutgoingContentsList(outgoingContentsVOS);
        }
        return deliveryRegistrationFormVOById;
    }

    @Override//通过输入领用码的方式查询出库内容
    public DeliveryRegistrationFormVO getDeliveryRegistrationFormVOByCode(String claimCode) {

        if (!StrUtil.isNotBlank(claimCode)) {
            return null;
        }
        ApplicationForUse applicationForUse = applicationForUseService.getOne(Wrappers.<ApplicationForUse>query()
                .eq("claim_code", claimCode));
        if (applicationForUse == null) {
            return null;
        }
        DeliveryRegistrationFormVO deliveryRegistrationFormVOById = baseMapper.getDeliveryRegistrationFormVOById(applicationForUse.getDeliveryRegistrationFormId());
        LambdaQueryWrapper<OutgoingContents> outgoingContentsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        outgoingContentsLambdaQueryWrapper.eq(OutgoingContents::getDeliveryRegistrationFormId, deliveryRegistrationFormVOById.getId());
        List<OutgoingContents> list = outgoingContentsService.list(outgoingContentsLambdaQueryWrapper);
        List<OutgoingContentsVO> outgoingContentsVOS = new ArrayList<>();
        if (list.size() != 0) {
            //为出库明细关联试剂耗材信息与供应商信息
            for (OutgoingContents outgoingContents : list) {
                OutgoingContentsVO outgoingContentsVO = new OutgoingContentsVO();
                BeanUtils.copyProperties(outgoingContents, outgoingContentsVO);
                LambdaQueryWrapper<ReagentConsumableInventory> reagentConsumableInventoryLambdaQueryWrapper = new LambdaQueryWrapper<>();
                reagentConsumableInventoryLambdaQueryWrapper.eq(ReagentConsumableInventory::getReagentConsumableId, outgoingContents.getReagentConsumableId());
                ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(reagentConsumableInventoryLambdaQueryWrapper);
                outgoingContentsVO.setReagentConsumableInventory(one);
                outgoingContentsVOS.add(outgoingContentsVO);
            }
            deliveryRegistrationFormVOById.setOutgoingContentsList(outgoingContentsVOS);
        }
        return deliveryRegistrationFormVOById;
    }

}
