package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.io.IoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import digital.laboratory.platform.common.feign.RemoteWord2PDFService;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.oss.service.OssFile;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.StandardMaterialApprovalFormDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.StandardMaterialApprovalFormMapper;
import digital.laboratory.platform.reagent.service.ReagentConsumableInventoryService;
import digital.laboratory.platform.reagent.service.ReferenceMaterialService;
import digital.laboratory.platform.reagent.service.StandardMaterialApprovalFormService;
import digital.laboratory.platform.reagent.service.SupplierInformationService;
import digital.laboratory.platform.reagent.vo.StandardMaterialApprovalFormVO;
import feign.Response;
import io.seata.spring.annotation.GlobalTransactional;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.time.LocalDateTime;
import java.util.Calendar;
import java.util.HashMap;

/**
 * (标准物质停用/报废销毁/恢复/降级使用审批表)服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (标准物质停用 / 报废销毁 / 恢复 / 降级使用审批表) 服务实现类
 */
@Service
@SuppressWarnings("all")
public class StandardMaterialApprovalFormServiceImpl extends ServiceImpl<StandardMaterialApprovalFormMapper, StandardMaterialApprovalForm> implements StandardMaterialApprovalFormService {
    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private OssFile ossFile;

    @Autowired
    private RemoteWord2PDFService remoteWord2PDFService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Override//提交表单
    public StandardMaterialApprovalForm commitById(StandardMaterialApprovalFormDTO standardMaterialApprovalFormDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm standardMaterialApprovalForm = new StandardMaterialApprovalForm();

        BeanUtils.copyProperties(standardMaterialApprovalFormDTO, standardMaterialApprovalForm);

        ReferenceMaterial referenceMaterialServiceById = referenceMaterialService.getById(standardMaterialApprovalFormDTO.getReferenceMaterialId());

//        if (referenceMaterialServiceById.getStatus()==-3){
//
//            throw new RuntimeException(String.format("该标准物质/溶液已被领用，请归还后，再进行操作"));
//        }

        standardMaterialApprovalForm.setApplicantId(dlpUser.getId());
        standardMaterialApprovalForm.setApplicantName(dlpUser.getName());
        standardMaterialApprovalForm.setCommitStatus(1);
        standardMaterialApprovalForm.setId(IdWorker.get32UUID().toUpperCase());
        standardMaterialApprovalForm.setCommitTime(LocalDateTime.now());


        if (this.save(standardMaterialApprovalForm)) {
            return standardMaterialApprovalForm;
        } else throw new RuntimeException(String.format("保存失败"));
    }


    @Override
    public IPage<StandardMaterialApprovalFormVO> getVOPage(IPage<StandardMaterialApprovalForm> page, QueryWrapper<StandardMaterialApprovalForm> qw) {

        IPage<StandardMaterialApprovalFormVO> voPage = baseMapper.getVOPage(page, qw);

        return voPage;
    }

    @Override
    public StandardMaterialApprovalFormVO getVO(String standardMaterialApprovalFormId) {

        StandardMaterialApprovalFormVO vo = baseMapper.getVO(standardMaterialApprovalFormId);

        SupplierInformation byId1 = supplierInformationService.getById(vo.getManufacturerId());

        vo.setManufacturerName(byId1.getSupplierName());

        return vo;
    }


    @Override//审核
    public StandardMaterialApprovalForm auditSecondary(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm byId = this.getById(auditAndApproveDTO.getUuId());

        byId.setAuditOpinionOfSecondary(auditAndApproveDTO.getAuditOpinion());
        byId.setAuditResultOfSecondary(auditAndApproveDTO.getAuditResult());
        byId.setSecondaryAuditorId(dlpUser.getId());
        byId.setAuditTimeOfSecondary(LocalDateTime.now());
        if (auditAndApproveDTO.getAuditResult() == true) {
            byId.setCommitStatus(4);
        } else {
            byId.setCommitStatus(-4);
        }
        if (this.updateById(byId)) {
            return byId;
        } else throw new RuntimeException(String.format("审核失败"));
    }

    @Override//审批
    @GlobalTransactional
    public StandardMaterialApprovalForm approveById(AuditAndApproveDTO auditAndApproveDTO, DLPUser dlpUser) {

        StandardMaterialApprovalForm byId1 = this.getById(auditAndApproveDTO.getUuId());

        byId1.setOpinionOfApproval(auditAndApproveDTO.getApproveOpinion());
        byId1.setResultOfApproval(auditAndApproveDTO.getApproveResult());
        byId1.setApproverId(dlpUser.getId());
        byId1.setApprovalOfTime(LocalDateTime.now());
        if (auditAndApproveDTO.getApproveResult() == true) {

            if (byId1.getApplicationContent() == 1 ) {

                ReferenceMaterial byId = referenceMaterialService.getById(byId1.getReferenceMaterialId());

                byId.setStatus(-1);
                byId.setStatus(-1);

                referenceMaterialService.updateById(byId);

            } else if (byId1.getApplicationContent() == 4) {

                ReferenceMaterial byId = referenceMaterialService.getById(byId1.getReferenceMaterialId());

                byId.setStatus(-2);
                byId.setServiceStatus(-2);

                referenceMaterialService.updateById(byId);


            }else if (byId1.getApplicationContent()==2){
                ReferenceMaterial byId = referenceMaterialService.getById(byId1.getReferenceMaterialId());
                byId.setStatus(-4);
                reagentConsumableInventoryService.updateCabinet(byId.getLatticeId(),byId.getId(),-1);
                referenceMaterialService.updateById(byId);
            }
            else {

                ReferenceMaterial byId = referenceMaterialService.getById(byId1.getReferenceMaterialId());
                byId.setStatus(0);
                byId.setServiceStatus(0);
                referenceMaterialService.updateById(byId);

            }
            byId1.setCommitStatus(5);
        } else {
            byId1.setCommitStatus(-5);
        }
        if (this.updateById(byId1)) {
            return byId1;
        } else throw new RuntimeException(String.format("审批失败"));
    }

    @Override
    public void standardMaterialApprovalFormTablePDF(StandardMaterialApprovalFormVO standardMaterialApprovalFormVO, HttpServletRequest theHttpServletRequest, HttpServletResponse httpServletResponse) throws Exception {
        System.out.println("standardMaterialApprovalFormTablePDF.................");

        String id = standardMaterialApprovalFormVO.getId();

        String reasonForApplication = standardMaterialApprovalFormVO.getReasonForApplication();

        Integer reasonForApplicationPrint = standardMaterialApprovalFormVO.getReasonForApplicationPrint();

        if (reasonForApplication.equals("发现质变或有异常,暂停使用")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(0);
        }
        if (reasonForApplication.equals("超过有效期或保质期,暂停使用")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(1);

        }
        if (reasonForApplication.equals("定值合格恢复使用")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(2);

        }
        if (reasonForApplication.equals("定值未达原等级要求,降级使用")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(3);

        }
        if (reasonForApplication.equals("定值未达使用要求，报废销毁")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(4);

        }
        if (reasonForApplication.equals("其他")) {
            standardMaterialApprovalFormVO.setReasonForApplicationPrint(5);

        }


        String applyFileName = "标准物质停用报废销毁恢复降级使用审批表" + id;
        //-----------------------------
        // 生成 word 版本的 验收记录表

        ByteArrayOutputStream bos = new ByteArrayOutputStream();

        ossFile.fileGet("template" + "/" + "标准物质停用报废销毁恢复降级使用审批表模板.docx", bos);

        byte[] templateArray = bos.toByteArray();

        ByteArrayInputStream bis = new ByteArrayInputStream(templateArray);

        bos.close();

        LoopRowTableRenderPolicy policy = new LoopRowTableRenderPolicy();

        ConfigureBuilder builder = Configure.builder().buildGramer("${", "}").useSpringEL(false);

        builder.bind("voList", policy).build();

        Calendar instance = Calendar.getInstance();
        int year = instance.get(Calendar.YEAR);
        /**
         * 循环表格
         */
        XWPFTemplate template = XWPFTemplate.compile(bis, builder.build()).render(
                new HashMap<String, Object>() {{
                    put("year", year);
                    put("referenceMaterialName",standardMaterialApprovalFormVO.getReferenceMaterialName());
                    put("referenceMaterialNumber",standardMaterialApprovalFormVO.getReferenceMaterialNumber());
                    put("batchNumber",standardMaterialApprovalFormVO.getBatchNumber());
                    put("specification",standardMaterialApprovalFormVO.getSpecification());
                    put("manufacturerName",standardMaterialApprovalFormVO.getManufacturerName());
                    put("purchaseTime", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getProcessingTime(),"yyyy年MM月dd日"));
                    put("commitTime", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getCommitTime(),"yyyy年MM月dd日"));
                    put("processingTime", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getProcessingTime(),"yyyy年MM月dd日"));
                    put("auditTimeOfPrimary", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getCommitTime(),"yyyy年MM月dd日"));
                    put("auditTimeOfSecondary", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getAuditTimeOfSecondary(),"yyyy年MM月dd日"));
                    put("approvalOfTime", LocalDateTimeUtil.format(standardMaterialApprovalFormVO.getApprovalOfTime(),"yyyy年MM月dd日"));
                    put("quantity", standardMaterialApprovalFormVO.getQuantity());
                    put("fixedResult", standardMaterialApprovalFormVO.getFixedResult());
                    put("createName", standardMaterialApprovalFormVO.getCreateName());
                    put("applicationContent", standardMaterialApprovalFormVO.getApplicationContent());
                    put("agentName", standardMaterialApprovalFormVO.getAgentName());
                    put("reasonForApplicationPrint",standardMaterialApprovalFormVO.getReasonForApplicationPrint());
                    put("auditOpinionOfPrimary",standardMaterialApprovalFormVO.getAuditOpinionOfPrimary());
                    put("primaryAuditorName",standardMaterialApprovalFormVO.getApplicantName());
                    put("auditOpinionOfSecondary",standardMaterialApprovalFormVO.getAuditOpinionOfSecondary());
                    put("secondaryAuditorName",standardMaterialApprovalFormVO.getSecondaryAuditorName());
                    put("opinionOfApproval",standardMaterialApprovalFormVO.getOpinionOfApproval());
                    put("approverName",standardMaterialApprovalFormVO.getApproverName());
                    put("remarks",standardMaterialApprovalFormVO.getRemarks());
                }}
        );
        bis.close();

        ByteArrayOutputStream fosWord = new ByteArrayOutputStream();
        template.write(fosWord);
        template.close();

        //------------
        ByteArrayInputStream fisWord = new ByteArrayInputStream(fosWord.toByteArray());
        fosWord.close();

        //MockMultipartFile mockMultipartFile = new MockMultipartFile("file", entrustmentLetterFileName + ".docx", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", fisWord);
        MockMultipartFile mockMultipartFile = new MockMultipartFile("file", applyFileName + ".docx", "image/jpg", fisWord);
        Response response = remoteWord2PDFService.word2pdf(mockMultipartFile);
        fisWord.close();


        ByteArrayOutputStream outPDF = new ByteArrayOutputStream();
        IoUtil.copy(response.body().asInputStream(), outPDF, IoUtil.DEFAULT_MIDDLE_BUFFER_SIZE);
        ByteArrayInputStream isPDF = new ByteArrayInputStream(outPDF.toByteArray());
        outPDF.close();


        ossFile.fileSave("document" + "/" + "standardMaterialApprovalForm" + "/" + id + "/" + applyFileName + ".pdf", isPDF);
        isPDF.close();


        System.out.println(String.format("转换为 PDF 结束"));

    }
}
