package digital.laboratory.platform.reagent.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.WarehousingRecordFormDTO;
import digital.laboratory.platform.reagent.entity.*;
import digital.laboratory.platform.reagent.mapper.WarehousingRecordFormMapper;
import digital.laboratory.platform.reagent.service.*;
import digital.laboratory.platform.reagent.vo.*;
import digital.laboratory.platform.sys.entity.CellAndStoreSupplyData;
import digital.laboratory.platform.sys.feign.RemoteCabinetService;
//import io.seata.spring.annotation.GlobalTransactional;
import io.seata.spring.annotation.GlobalTransactional;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * 签收记录表服务实现类
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收记录表 服务实现类
 */
@Service
@SuppressWarnings("all")
public class WarehousingRecordFormServiceImpl extends ServiceImpl<WarehousingRecordFormMapper, WarehousingRecordForm> implements WarehousingRecordFormService {
    @Autowired
    private WarehousingContentService warehousingContentService;

    @Autowired
    private WarehousingBatchListService warehousingBatchListService;

    @Autowired
    private ReagentConsumableInventoryService reagentConsumableInventoryService;

    @Autowired
    private ReagentConsumablesService reagentConsumablesService;
    @Autowired
    private ReferenceMaterialService referenceMaterialService;

    @Autowired
    private RemoteCabinetService remoteCabinetService;

    @Autowired
    private BatchDetailsService batchDetailsService;

    @Autowired
    private AcceptanceRecordFormService acceptanceRecordFormService;
    @Autowired
    private ProvideServicesOrSuppliesService provideServicesOrSuppliesService;

    @Autowired
    private EvaluationFormService evaluationFormService;

    @Autowired
    private SupplierInformationService supplierInformationService;

    @Autowired
    private PurchaseListDetailsService purchaseListDetailsService;

    @Autowired
    private ProcurementContentService procurementContentService;
    @Autowired

    private DetailsOfCentralizedService detailsOfCentralizedService;

    @Autowired
    private DecentralizeDetailsService decentralizeDetailsService;

    @Autowired
    private PurchasingPlanService purchasingPlanService;

    @Autowired
    private CentralizedRequestService centralizedRequestService;

    @Autowired
    private DecentralizedRequestService decentralizedRequestService;

    @Autowired
    private PurchaseListService purchaseListService;
    @Autowired
    private AcceptanceSignForService acceptanceSignForService;

    @Autowired
    private ReagentConsumableStashService reagentConsumableStashService;

    @GlobalTransactional//查看采购入库
    public WarehousingRecordFormVO getWarehousingRecordFormVO(String warehousingFormId) {

        WarehousingRecordForm byId = this.getById(warehousingFormId);
        WarehousingRecordFormVO warehousingRecordFormVO = new WarehousingRecordFormVO();
        BeanUtils.copyProperties(byId, warehousingRecordFormVO);
        //查询出签收物品明细，将明细添加至签收记录VO
        warehousingRecordFormVO.setWarehousingContentVOList(warehousingContentService.getWarehousingContentVOList(warehousingFormId));
        return warehousingRecordFormVO;
    }


    @GlobalTransactional
    @Override//录入入库明细
    public void addFormById(WarehousingRecordFormDTO warehousingRecordFormDTO, DLPUser dlpUser) {

        if (warehousingRecordFormDTO.getDateOfProduction() == null) {//如果没有生产日期，默认当前时间为生产日期
            warehousingRecordFormDTO.setDateOfProduction(LocalDate.now());
        }
        if ((warehousingRecordFormDTO.getWarningValue() == null)) {
            warehousingRecordFormDTO.setWarningValue(1);
        }
        //必须选择存放位置才能入库
        if (warehousingRecordFormDTO.getLatticeId().isEmpty() & warehousingRecordFormDTO.getLocation().isEmpty() &
                warehousingRecordFormDTO.getBoxId().isEmpty()) {
            throw new RuntimeException(String.format("请选择存放位置后再进行提交"));
        }

        String latticeId = warehousingRecordFormDTO.getLatticeId();//格子ID
        String boxId = warehousingRecordFormDTO.getBoxId();//柜子ID
        String location = warehousingRecordFormDTO.getLocation();//位置信息

        String roomNo = location.substring(3, 6);
        //建立签收物品批次信息
        WarehousingBatchList warehousingBatchList = new WarehousingBatchList();
        BeanUtils.copyProperties(warehousingRecordFormDTO, warehousingBatchList);
        warehousingBatchList.setId(IdWorker.get32UUID().toUpperCase());
        warehousingBatchList.setDepositorId(dlpUser.getId());//签收人
        warehousingBatchList.setDateOfReceipt(LocalDateTime.now());//签收日期
        warehousingBatchList.setLocation(location);//位置信息

        WarehousingContent warehousingContent = warehousingContentService.getById(warehousingBatchList.getWarehousingContentId());//签收内容对象

        ReagentConsumables reagentConsumables = reagentConsumablesService.getById(warehousingContent.getReagentConsumableId());//得到试剂耗材对象

        Integer quantity = warehousingRecordFormDTO.getQuantity() * Integer.valueOf(reagentConsumables.getPackagedCopies());//入库数量=购买数量x包装份数

        //判断采购数量与签收数量
        if (warehousingContent.getWarehousingQuantity() > warehousingContent.getTotalQuantity()) {
            throw new RuntimeException(String.format("入库数量不能大于采购数量"));
        }

        warehousingBatchList.setBatch(warehousingBatchListService.list(Wrappers.<WarehousingBatchList>query().eq("warehousing_content_id", warehousingBatchList.getWarehousingContentId())).size() + 1);


        warehousingContent.setLastStorageLocation(warehousingRecordFormDTO.getLocation());//记录存放位置信息，以便下次签收同样物品时可以回显位置信息
        warehousingContent.setBoxId(boxId);//柜子ID
        warehousingContent.setLatticeId(latticeId);//格子ID

        warehousingContent.setWarehousingQuantity(warehousingContent.getWarehousingQuantity() + warehousingBatchList.getQuantity());//修改签收数量

        if (warehousingRecordFormDTO.getCode() != null) {//如果填入物品编码，则记录，以便下次签收回显
            warehousingContent.setCode(warehousingRecordFormDTO.getCode());
        }

        ReagentConsumableInventory one = reagentConsumableInventoryService.getOne(Wrappers.<ReagentConsumableInventory>query().eq("reagent_consumable_id", warehousingContent.getReagentConsumableId()));
        //录入仓库批次信息，若仓库不存在该物品，则新增该物品的库存信息
        if (one == null) {

            ReagentConsumableInventory reagentConsumableInventory = new ReagentConsumableInventory();//创建库存对象


            BeanUtils.copyProperties(reagentConsumables, reagentConsumableInventory);//拷贝信息

            reagentConsumableInventory.setReagentConsumableInventoryId(IdWorker.get32UUID().toUpperCase());
            reagentConsumableInventory.setStatus(1);//状态 1为正常
            reagentConsumableInventory.setCreateTime(LocalDateTime.now());
            reagentConsumableInventory.setUpdateTime(LocalDateTime.now());
            reagentConsumableInventory.setWarningValue(warehousingRecordFormDTO.getWarningValue());//预警值
            //如果供应商未提供物品编码，则需在入库时扫描标签录入编码
            if (warehousingRecordFormDTO.getCode() != null) {
                reagentConsumableInventory.setCode(warehousingRecordFormDTO.getCode());
                reagentConsumables.setCode(warehousingRecordFormDTO.getCode());
            } else {
                reagentConsumableInventory.setCode("");
                reagentConsumables.setCode("");
            }
            reagentConsumablesService.updateById(reagentConsumables);

            BatchDetails batchDetails = new BatchDetails();//创建批次信息

            int year = Calendar.getInstance().get(Calendar.YEAR);//获取当前年

            BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);//拷贝批次信息
            batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
            batchDetails.setServiceStatus(1);//状态 1为正常
            batchDetails.setPurchaseTime(LocalDateTime.now());//购买时间
            batchDetails.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());//关联库存物品ID
            batchDetails.setSupplierId(warehousingContent.getSupplierId());//供应商ID
            batchDetails.setBatch(year + "-" + 1);//由于是第一次入库该物品，所以批次肯定为1
            batchDetails.setLatticeId(latticeId);//赋值位置信息
            batchDetails.setLocation(location);
            batchDetails.setBoxId(boxId);
            batchDetails.setRoomNo(roomNo);//房间号
            batchDetails.setReagentConsumableName(reagentConsumables.getReagentConsumableName());
            //如果未填写存储期限，则默认为可以一直存储
            if (warehousingRecordFormDTO.getExpirationDate() != null) {
                batchDetails.setExpirationDate(warehousingRecordFormDTO.getExpirationDate());
            } else {
                batchDetails.setExpirationDate(LocalDate.now().plusYears(99));
            }

            batchDetails.setQuantity(quantity); //录入数量
            batchDetails.setWarehousingBatchListId(warehousingBatchList.getId());//关联签收批次

            if (!reagentConsumables.getCategory().equals("标准物质")) {

                ReagentConsumableStash reagentConsumableStash = new ReagentConsumableStash();
                BeanUtils.copyProperties(reagentConsumableInventory, reagentConsumableStash);
                reagentConsumableStash.setId(IdWorker.get32UUID().toUpperCase());
                reagentConsumableStash.setRoomNo(roomNo);
                reagentConsumableStash.setReagentConsumableId(reagentConsumables.getReagentConsumableId());
                batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                reagentConsumableStash.setTotalQuantity(quantity);
                reagentConsumableStashService.save(reagentConsumableStash);
            }

            warehousingBatchList.setBatchId(batchDetails.getBatchDetailsId());//关联签收批次

            if (reagentConsumables.getCategory().equals("试剂") | reagentConsumables.getCategory().equals("耗材")) {
                //更新格子信息（试剂耗材传入类ID）
                reagentConsumableInventoryService.updateCabinet(batchDetails.getLatticeId(), reagentConsumables.getReagentConsumableId(), 1);
            }
            reagentConsumableInventory.setTotalQuantity(quantity);//数量

            //如果是标准物质，则需生成编号
            if (reagentConsumables.getCategory().equals("标准物质")) {
                //需生成对应购买数量的标准物质以及编号
                for (int j = 0; j < batchDetails.getQuantity(); j++) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setReagentConsumableId(reagentConsumables.getReagentConsumableId());//试剂耗材ID
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());//批次ID
                    referenceMaterial.setReagentConsumableInventoryId(reagentConsumableInventory.getReagentConsumableInventoryId());//库存物品ID
                    referenceMaterial.setStatus(0);//状态 0为正常
                    referenceMaterial.setLocation(location);
                    referenceMaterial.setLatticeId(latticeId);
                    referenceMaterial.setBoxId(boxId);
                    referenceMaterial.setReagentConsumableName(reagentConsumables.getReagentConsumableName());
                    referenceMaterial.setMass(warehousingRecordFormDTO.getReferenceMaterialMass());
                    //更新格子信息（标准物质传入对象ID）
                    reagentConsumableInventoryService.updateCabinet(referenceMaterial.getLatticeId(), referenceMaterial.getId(), 1);
                    //生成编号
                    String prefix = reagentConsumables.getAlias() + "-" + year + "-" + location.charAt(1) + "-";
                    List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                            .likeRight("number", prefix)
                            .orderByDesc("number"));
                    int newNo = 1;
                    if ((list != null) && (list.size() > 0)) {
                        ReferenceMaterial referenceMaterial1 = list.get(0);
                        String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                        try {
                            int maxno = Integer.parseUnsignedInt(strMaxNo);
                            newNo = maxno + 1;
                        } catch (NumberFormatException e) {
                            // 如果后缀有非数字, 则无视之, 重头编码
                            newNo = 1;
                        }
                    }
                    referenceMaterial.setNumber(prefix + newNo);
                    referenceMaterialService.save(referenceMaterial);
                }
            }
            reagentConsumableInventoryService.save(reagentConsumableInventory);
            batchDetailsService.save(batchDetails);
        } else {
            //如果仓库存在该物品
            if (warehousingRecordFormDTO.getCode() != null) {
                one.setCode(warehousingRecordFormDTO.getCode());
                reagentConsumables.setCode(warehousingContent.getCode());
                reagentConsumablesService.updateById(reagentConsumables);
            }
            BatchDetails batchDetails = new BatchDetails();
            BeanUtils.copyProperties(warehousingRecordFormDTO, batchDetails);
            batchDetails.setBatchDetailsId(IdWorker.get32UUID().toUpperCase());
            batchDetails.setServiceStatus(1);
            batchDetails.setPurchaseTime(LocalDateTime.now());
            batchDetails.setSupplierId(warehousingContent.getSupplierId());
            //如果未填写存储期限，则默认为可以一直存储
            if (warehousingRecordFormDTO.getExpirationDate() != null) {
                batchDetails.setExpirationDate(warehousingRecordFormDTO.getExpirationDate());
            } else {
                batchDetails.setExpirationDate(LocalDate.now().plusYears(99));
            }
            batchDetails.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
            batchDetails.setQuantity(quantity);
            batchDetails.setWarehousingBatchListId(warehousingBatchList.getId());
            warehousingBatchList.setBatchId(batchDetails.getBatchDetailsId());
            batchDetails.setLocation(location);
            batchDetails.setLatticeId(latticeId);
            batchDetails.setBoxId(boxId);
            batchDetails.setRoomNo(roomNo);//房间号
            batchDetails.setReagentConsumableName(reagentConsumables.getReagentConsumableName());

            if (!reagentConsumables.getCategory().equals("标准物质")) {

                ReagentConsumableStash reagentConsumableStash = reagentConsumableStashService.getOne(Wrappers.<ReagentConsumableStash>query()
                        .eq("reagent_consumable_id", reagentConsumables.getReagentConsumableId())
                        .eq("room_no", roomNo));
                if (reagentConsumableStash == null) {
                    ReagentConsumableStash newReagentConsumableStash = new ReagentConsumableStash();
                    BeanUtils.copyProperties(one, newReagentConsumableStash);
                    newReagentConsumableStash.setId(IdWorker.get32UUID().toUpperCase());
                    newReagentConsumableStash.setRoomNo(roomNo);
                    newReagentConsumableStash.setTotalQuantity(quantity);
                    batchDetails.setReagentConsumableStashId(newReagentConsumableStash.getId());
                    reagentConsumableStashService.save(newReagentConsumableStash);
                } else {
                    reagentConsumableStash.setTotalQuantity(reagentConsumableStash.getTotalQuantity() + quantity);
                    batchDetails.setReagentConsumableStashId(reagentConsumableStash.getId());
                    reagentConsumableStashService.updateById(reagentConsumableStash);
                }
            }
            int year = Calendar.getInstance().get(Calendar.YEAR);

            Integer years = Integer.valueOf(year);//进行包装，方便运算
            //查询相同年份相同供应商相同物品的所有批次信息
            List<BatchDetails> batchDetailsList = batchDetailsService.list(Wrappers.<BatchDetails>query().eq("reagent_consumable_inventory_id", one.getReagentConsumableInventoryId())
                    .eq("supplier_id", warehousingContent.getSupplierId())
                    .like("batch", years));
            //如果没有批次信心，则默认为1
            if (batchDetailsList.size() == 0) {
                batchDetails.setBatch(year + "-" + 1);
            } else {
                Integer x = batchDetailsList.size() + 1;
                batchDetails.setBatch(year + "-" + x);
            }
            if (one.getCategory().equals("试剂") | one.getCategory().equals("耗材")) {
                //更新格子信息（试剂耗材传入类ID）
                reagentConsumableInventoryService.updateCabinet(batchDetails.getLatticeId(), reagentConsumables.getReagentConsumableId(), 1);
            }
            one.setTotalQuantity(one.getTotalQuantity() + batchDetails.getQuantity());
            one.setWarningValue(warehousingRecordFormDTO.getWarningValue());

            if (one.getCategory().equals("标准物质")) {
                for (int j = 0; j < batchDetails.getQuantity(); j++) {
                    ReferenceMaterial referenceMaterial = new ReferenceMaterial();
                    referenceMaterial.setId(IdWorker.get32UUID().toUpperCase());
                    referenceMaterial.setReagentConsumableId(one.getReagentConsumableId());
                    referenceMaterial.setBatchDetailsId(batchDetails.getBatchDetailsId());
                    referenceMaterial.setReagentConsumableInventoryId(one.getReagentConsumableInventoryId());
                    referenceMaterial.setStatus(0);
                    referenceMaterial.setLocation(warehousingRecordFormDTO.getLocation());
                    referenceMaterial.setLatticeId(warehousingRecordFormDTO.getLatticeId());
                    referenceMaterial.setBoxId(warehousingRecordFormDTO.getBoxId());
                    referenceMaterial.setCustodianName(warehousingRecordFormDTO.getCustodianName());
                    referenceMaterial.setReagentConsumableName(reagentConsumables.getReagentConsumableName());
                    referenceMaterial.setMass(warehousingRecordFormDTO.getReferenceMaterialMass());
                    //更新格子信息（试剂耗材传入类ID）
                    reagentConsumableInventoryService.updateCabinet(referenceMaterial.getLatticeId(), referenceMaterial.getId(), 1);
                    //生成编号
                    String prefix = reagentConsumables.getAlias() + "-" + year + "-" + location.charAt(1) + "-";
                    List<ReferenceMaterial> list = referenceMaterialService.list(Wrappers.<ReferenceMaterial>query()
                            .likeRight("number", prefix)
                            .orderByDesc("number"));
                    int newNo = 1;
                    if ((list != null) && (list.size() > 0)) {
                        ReferenceMaterial referenceMaterial1 = list.get(0);
                        String strMaxNo = StrUtil.removePrefixIgnoreCase(referenceMaterial1.getNumber(), prefix);
                        try {
                            int maxno = Integer.parseUnsignedInt(strMaxNo);
                            newNo = maxno + 1;
                        } catch (NumberFormatException e) {
                            // 如果后缀有非数字, 则无视之, 重头编码
                            newNo = 1;
                        }
                    }
                    referenceMaterial.setNumber(prefix + newNo);
                    referenceMaterialService.save(referenceMaterial);
                }

            }
            reagentConsumableInventoryService.updateById(one);
            batchDetailsService.save(batchDetails);
        }

        warehousingContentService.updateById(warehousingContent);
        warehousingBatchListService.save(warehousingBatchList);

        //至此，签收结束，下面将判断签收单是否完成
        String warehousingRecordFormId = warehousingContent.getWarehousingRecordFormId();
        List<WarehousingContent> list = warehousingContentService.list(Wrappers.<WarehousingContent>query().eq("warehousing_record_form_id", warehousingRecordFormId));
        WarehousingRecordForm warehousingRecordForm = this.getById(warehousingRecordFormId);

        PurchaseList purchaseList = purchaseListService.getById(purchaseListDetailsService.getById(warehousingContent.getPurchaseListDetailsId()).getPurchaseListId());//查询采购清单
        int i = 0;

//遍历采购内容，判断试剂耗材是否入库完毕
        for (WarehousingContent warehousingContentFull : list) {

            if (warehousingContentFull.getWarehousingQuantity() == warehousingContentFull.getTotalQuantity()) {//签收数量=购买数量

                warehousingContentFull.setStatus(1);//签收成功

                warehousingContentService.updateById(warehousingContentFull);

                if (warehousingContentFull.getAcceptanceRecordFormId() == null) {//如果没有验收记录表，那就创建
                    //首先创建验收集合表
                    AcceptanceSignFor acceptanceSignForServiceOne = acceptanceSignForService.getOne(Wrappers.<AcceptanceSignFor>query().eq("warehousing_record_form_id", warehousingRecordFormId));
                    //首次创建
                    if (acceptanceSignForServiceOne == null) {
                        AcceptanceSignFor acceptanceSignFor = new AcceptanceSignFor();
                        acceptanceSignFor.setWarehousingRecordFormId(warehousingRecordFormId);
                        acceptanceSignFor.setId(IdWorker.get32UUID().toUpperCase());
                        acceptanceSignFor.setQuantity(list.size());
                        acceptanceSignFor.setStatus(1);
                        acceptanceSignFor.setCreateTime(purchaseList.getCreateTime());
                        acceptanceSignForService.save(acceptanceSignFor);
                        AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.addForm(warehousingContent.getReagentConsumableId(), warehousingContent.getSupplierId(), acceptanceSignFor.getId());
                        warehousingContent.setAcceptanceRecordFormId(acceptanceRecordForm.getId());
                    } else {
                        AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.addForm(warehousingContent.getReagentConsumableId(), warehousingContent.getSupplierId(), acceptanceSignForServiceOne.getId());
                        warehousingContent.setAcceptanceRecordFormId(acceptanceRecordForm.getId());
                    }

                    warehousingContentService.updateById(warehousingContent);
                }

                i = i + 1;
            }
        }
        //判断该清单所有试剂耗材是否入库完成,若入库完成，则需生成供应商评价表
        if (i == list.size()) {//如果签收完毕
            List<SupplierInformation> supplierInformations = new ArrayList<>();//供应商集合
            //创建供应商评价表
            /*
            通过循环签收内容，获得所有的供应商列表
            为每一个供应商创建一个评价表
            */
            for (WarehousingContent warehousingContentFull : list) {//遍历签收物品明细
                SupplierInformation supplierInformation = supplierInformationService.getById(warehousingContentFull.getSupplierId());//供应商
                if (!supplierInformations.contains(supplierInformation)) {
                    supplierInformations.add(supplierInformation);//将所有供应商添加至集合
                    EvaluationForm evaluationForm = evaluationFormService.addForm(supplierInformation.getId());//创建供应商评价表
                    warehousingContentFull.setEvaluationFormId(evaluationForm.getId());//关联供应商与签收明细
                    provideServicesOrSuppliesService.addById(evaluationForm.getId(), warehousingContentFull.getReagentConsumableId());//创建供应商提供物品信息
                } else {
                    //如果是相同供应商的物品
                    LambdaQueryWrapper<EvaluationForm> evaluationFormLambdaQueryWrapper = new LambdaQueryWrapper<>();
                    evaluationFormLambdaQueryWrapper.eq(EvaluationForm::getSupplierInformationId, supplierInformation.getId())
                            .orderByDesc(EvaluationForm::getCreateTime);
                    List<EvaluationForm> list1 = evaluationFormService.list(evaluationFormLambdaQueryWrapper);
                    provideServicesOrSuppliesService.addById(list1.get(0).getId(), warehousingContentFull.getReagentConsumableId());//为最新创建的评价表添加物品信息
                }
            }
            warehousingRecordForm.setStatus(2);//改变签收单状态
        } else {
            warehousingRecordForm.setStatus(1);
        }
        this.updateById(warehousingRecordForm);

    }
    @Override
    public IPage<WarehousingRecordFormVO> getWarehousingRecordFormVOPage(Page<WarehousingRecordForm> page, QueryWrapper<WarehousingRecordForm> qw) {

        IPage<WarehousingRecordFormVO> warehousingRecordFormVOPage = baseMapper.getWarehousingRecordFormVOPage(page, qw);//查询签收单列表

        List<WarehousingRecordFormVO> records = warehousingRecordFormVOPage.getRecords();

        for (WarehousingRecordFormVO record : records) {

            List<WarehousingContent> warehousingContents = warehousingContentService.list(Wrappers.<WarehousingContent>query().eq("warehousing_record_form_id", record.getId()));
            //计算签收(购买)物品总数与已签收数量
            for (WarehousingContent warehousingContent : warehousingContents) {
                Integer warehousingQuantity = warehousingContent.getWarehousingQuantity();
                Integer totalQuantity = warehousingContent.getTotalQuantity();
                if (record.getWarehousingQuantity() == null & record.getQuantityPurchased() == null) {
                    record.setWarehousingQuantity(0);
                    record.setQuantityPurchased(0);
                }
                record.setWarehousingQuantity(record.getWarehousingQuantity() + warehousingQuantity);
                record.setQuantityPurchased(record.getQuantityPurchased() + totalQuantity);
            }
        }
        return warehousingRecordFormVOPage;
    }
}



